@@ -1,45 +1,40 @@
 import logging
+from typing import Any
 
+import motor.motor_asyncio
 from rest_tools.server import RestHandlerSetup, RestHandler
 
-import iceprod
+from iceprod.util import VERSION_STRING
+from iceprod.prom_utils import PromRequestMixin
 from .auth import AttrAuthMixin
 
 logger = logging.getLogger('rest')
 
 
-def IceProdRestConfig(config=None, statsd=None, database=None, auth_database=None, s3conn=None):
-    config['server_header'] = 'IceProd/' + iceprod.__version__
+DB = motor.motor_asyncio.AsyncIOMotorDatabase | motor.motor_asyncio.AsyncIOMotorClient
+
+
+def IceProdRestConfig(config: dict[str, Any], database: DB, auth_database=None, s3conn=None):
+    if config:
+        config['server_header'] = 'IceProd/' + VERSION_STRING
     ret = RestHandlerSetup(config)
-    ret['statsd'] = statsd
     ret['database'] = database
     ret['auth_database'] = auth_database
     ret['s3'] = s3conn
     return ret
 
 
-class APIBase(AttrAuthMixin, RestHandler):
+class APIBase(AttrAuthMixin, PromRequestMixin, RestHandler):
     """Default REST handler"""
-    def initialize(self, database=None, auth_database=None, statsd=None, s3=None, **kwargs):
-        super().initialize(**kwargs)
+    def initialize(self, *args, database, auth_database, s3=None, **kwargs):
+        super().initialize(*args, **kwargs)
         self.db = database
         self.auth_db = auth_database
-        self.statsd = statsd
         self.s3 = s3
 
-    def prepare(self):
-        super().prepare()
-        if self.statsd:
-            self.statsd.incr(f'prepare.{self.__class__.__name__}.{self.request.method}')
-
-    def on_finish(self):
-        super().on_finish()
-        if self.statsd:
-            self.statsd.incr(f'finish.{self.__class__.__name__}.{self.request.method}.{self.get_status()}')
-
     def get_template_namespace(self):
         namespace = super().get_template_namespace()
-        namespace['version'] = iceprod.__version__
+        namespace['version'] = VERSION_STRING
         return namespace
 
     def get_current_user(self):
