@@ -1,11 +1,13 @@
-import logging
-import json
-import uuid
-import math
 from collections import defaultdict
+import json
+import logging
+import math
+import re
+import uuid
 
 import pymongo
 import tornado.web
+from wipac_dev_tools import strtobool
 
 from ..base_handler import APIBase
 from ..auth import authorization, attr_auth
@@ -83,6 +85,7 @@ class MultiTasksHandler(APIBase):
 
         Params (optional):
             status: | separated list of task status to filter by
+            site: site to filter on
             keys: | separated list of keys to return for each task
             sort: | separated list of sort key=values, with values of 1 or -1
             limit: number of tasks to return
@@ -92,13 +95,14 @@ class MultiTasksHandler(APIBase):
         """
         filters = {}
 
-        status = self.get_argument('status', None)
-        if status:
+        if status := self.get_argument('status', None):
             filters['status'] = {'$in': status.split('|')}
 
-        sort = self.get_argument('sort', None)
+        if site := self.get_argument('site', None):
+            filters['site'] = {'$regex': '^'+re.escape(site)}
+
         mongo_sort = []
-        if sort:
+        if sort := self.get_argument('sort', None):
             for s in sort.split('|'):
                 if '=' in s:
                     name, order = s.split('=', 1)
@@ -109,8 +113,7 @@ class MultiTasksHandler(APIBase):
                 else:
                     mongo_sort.append((s, pymongo.ASCENDING))
 
-        limit = self.get_argument('limit', 0)
-        if limit:
+        if limit := self.get_argument('limit', 0):
             try:
                 limit = int(limit)
             except Exception:
@@ -306,44 +309,6 @@ class TasksStatusHandler(APIBase):
         self.finish()
 
 
-class TaskCountsStatusHandler(APIBase):
-    """
-    Handle task summary grouping by status.
-    """
-    @authorization(roles=['admin', 'system'])
-    async def get(self):
-        """
-        Get the task counts for all tasks, group by status.
-
-        Params (optional):
-            status: | separated list of task status to filter by
-
-        Returns:
-            dict: {<status>: num}
-        """
-        match = {}
-        status = self.get_argument('status', None)
-        if status:
-            status_list = status.split('|')
-            if any(s not in TASK_STATUS for s in status_list):
-                raise tornado.web.HTTPError(400, reaosn='Unknown task status')
-            match['status'] = {'$in': status_list}
-
-        ret = {}
-        cursor = self.db.tasks.aggregate([
-            {'$match': match},
-            {'$group': {'_id': '$status', 'total': {'$sum': 1}}},
-        ])
-        ret = {}
-        async for row in cursor:
-            ret[row['_id']] = row['total']
-        ret2 = {}
-        for k in sorted(ret, key=task_status_sort):
-            ret2[k] = ret[k]
-        self.write(ret2)
-        self.finish()
-
-
 class DatasetMultiTasksHandler(APIBase):
     """
     Handle multi tasks requests.
@@ -551,23 +516,11 @@ class DatasetTaskSummaryStatusHandler(APIBase):
         self.finish()
 
 
-class DatasetTaskCountsStatusHandler(APIBase):
+class TaskCountsStatusHandler(APIBase):
     """
     Handle task summary grouping by status.
     """
-    @authorization(roles=['admin', 'user', 'system'])
-    @attr_auth(arg='dataset_id', role='read')
-    async def get(self, dataset_id):
-        """
-        Get the task counts for all tasks in a dataset, group by status.
-
-        Args:
-            dataset_id (str): dataset id
-
-        Returns:
-            dict: {<status>: num}
-        """
-        match = {'dataset_id': dataset_id}
+    async def counts(self, match):
         status = self.get_argument('status', None)
         if status:
             status_list = status.split('|')
@@ -575,6 +528,13 @@ class DatasetTaskCountsStatusHandler(APIBase):
                 raise tornado.web.HTTPError(400, reaosn='Unknown task status')
             match['status'] = {'$in': status_list}
 
+        gpu = self.get_argument('gpu', None)
+        if gpu is not None:
+            if strtobool(gpu):
+                match['requirements.gpu'] = {'$gte': 1}
+            else:
+                match['$or'] = [{"requirements.gpu": {"$exists": False}}, {"requirements.gpu": {"$lte": 0}}]
+
         ret = {}
         cursor = self.db.tasks.aggregate([
             {'$match': match},
@@ -589,6 +549,44 @@ class DatasetTaskCountsStatusHandler(APIBase):
         self.write(ret2)
         self.finish()
 
+    @authorization(roles=['admin', 'system'])
+    async def get(self):
+        """
+        Get the task counts for all tasks, group by status.
+
+        Params (optional):
+            status: | separated list of task status to filter by
+            gpu: bool to select only gpu tasks or non-gpu tasks
+
+        Returns:
+            dict: {<status>: num}
+        """
+        await self.counts(match={})
+
+
+class DatasetTaskCountsStatusHandler(TaskCountsStatusHandler):
+    """
+    Handle task summary grouping by status.
+    """
+    @authorization(roles=['admin', 'user', 'system'])
+    @attr_auth(arg='dataset_id', role='read')
+    async def get(self, dataset_id):
+        """
+        Get the task counts for all tasks in a dataset, group by status.
+
+        Args:
+            dataset_id (str): dataset id
+
+        Params (optional):
+            status: | separated list of task status to filter by
+            gpu: bool to select only gpu tasks or non-gpu tasks
+
+        Returns:
+            dict: {<status>: num}
+        """
+        match = {'dataset_id': dataset_id}
+        await self.counts(match=match)
+
 
 class DatasetTaskCountsNameStatusHandler(APIBase):
     """
@@ -764,7 +762,6 @@ class TasksActionsQueueHandler(APIBase):
             logger.info('filter_query: %r', filter_query)
             self.send_error(404, reason="Task not found")
         else:
-            self.statsd.incr('site.{}.task_queued'.format(site))
             self.write(ret)
             self.finish()
 
@@ -905,25 +902,6 @@ class TasksActionsErrorHandler(APIBase):
             if 'site' in data:
                 site = data['site']
                 update_query['$set']['site'] = site
-            if self.statsd and 'reason' in data and data['reason']:
-                reason = 'other'
-                reasons = [
-                    ('Exception: failed to download', 'download_failure'),
-                    ('Exception: failed to upload', 'upload_failure'),
-                    ('Exception: module failed', 'module_failure'),
-                    ('Resource overusage for cpu', 'cpu_overuse'),
-                    ('Resource overusage for gpu', 'gpu_overuse'),
-                    ('Resource overusage for memory', 'memory_overuse'),
-                    ('Resource overusage for disk', 'disk_overuse'),
-                    ('Resource overusage for time', 'time_overuse'),
-                    ('pilot SIGTERM', 'sigterm'),
-                    ('killed', 'killed'),
-                ]
-                for text,r in reasons:
-                    if text in data['reason']:
-                        reason = r
-                        break
-                self.statsd.incr('site.{}.task_{}.{}'.format(site, self.final_status, reason))
 
         ret = await self.db.tasks.find_one_and_update(
             filter_query,
@@ -974,7 +952,7 @@ class TasksActionsCompleteHandler(APIBase):
 
         filter_query = {
             'task_id': task_id,
-            'status': 'processing',
+            'status': {'$in': task_prev_statuses('complete')},
             'instance_id': data['instance_id'],
         }
         update_query = {
@@ -987,11 +965,8 @@ class TasksActionsCompleteHandler(APIBase):
 
         if 'time_used' in data:
             update_query['$set']['walltime'] = data['time_used']/3600.
-        site = 'unknown'
         if 'site' in data:
-            site = data['site']
-            update_query['$set']['site'] = site
-        self.statsd.incr('site.{}.task_complete'.format(site))
+            update_query['$set']['site'] = data['site']
 
         ret = await self.db.tasks.find_one_and_update(
             filter_query,
