@@ -15,12 +15,16 @@ from typing import Any, Protocol
 from asyncache import cachedmethod  # type: ignore
 from cachetools import TTLCache
 from cachetools.func import ttl_cache
+from prometheus_client import Info
 import requests.exceptions
+from wipac_dev_tools.prometheus_tools import GlobalLabels, AsyncPromWrapper, AsyncPromTimer
 
+from iceprod.util import VERSION_STRING
 from iceprod.core import functions
 from iceprod.core.config import Task, Job, Dataset
 from iceprod.core.defaults import add_default_options
 from iceprod.core.resources import Resources
+from iceprod.prom_utils import HistogramBuckets
 from iceprod.server.states import JOB_STATUS_START
 from iceprod.server.util import nowstr
 
@@ -55,7 +59,7 @@ class BaseGrid:
         queue_cfg = self.cfg['queue']
 
         # site name
-        self.site = None
+        self.site = ''
         if 'site' in queue_cfg:
             self.site = queue_cfg['site']
 
@@ -84,15 +88,31 @@ class BaseGrid:
         # dataset lookup cache
         self.dataset_cache = TTLCache(maxsize=100, ttl=60)
 
+        i = Info('iceprod', 'IceProd information')
+        i.info({
+            'name': str(self.site),
+            'type': 'grid',
+            'queue_type': queue_cfg.get('type',''),
+            'version': VERSION_STRING,
+            'exclusive': str(queue_cfg.get('exclusive', False)),
+        })
+
+        self.prometheus = GlobalLabels({
+            'site': str(self.site),
+            'type': queue_cfg.get('type',''),
+            'gpu': str(self.site_requirements.get('gpu', 0)),
+        })
+
     # Functions to override #
 
     async def run(self):
         """Override the `ActiveJobs` and `JobActions` batch submission / monitoring classes"""
         raise NotImplementedError()
 
-    # Commong functions #
+    # Common functions #
 
     @cachedmethod(lambda self: self.dataset_cache)
+    @AsyncPromTimer(lambda self: self.prometheus.histogram('iceprod_grid_dataset_lookup', 'IceProd grid.dataset_lookup calls', buckets=HistogramBuckets.SECOND))
     async def dataset_lookup(self, dataset_id: str) -> Dataset:
         """
         Lookup a dataset fromn the REST API.
@@ -110,7 +130,9 @@ class BaseGrid:
         ret.validate()
         return ret
 
-    async def get_tasks_to_queue(self, num: int) -> list[Task]:
+    @AsyncPromWrapper(lambda self: self.prometheus.counter('iceprod_grid_queue_tasks', 'IceProd grid tasks queued', labels=['step'], finalize=False))
+    @AsyncPromTimer(lambda self: self.prometheus.histogram('iceprod_grid_tasks_to_queue', 'IceProd grid.tasks_to_queue calls'))
+    async def get_tasks_to_queue(self, prom_counter, num: int) -> list[Task]:
         """
         Get new tasks to queue from the REST API.
 
@@ -136,43 +158,61 @@ class BaseGrid:
                 raise
             else:
                 futures.add(asyncio.create_task(self._convert_to_task(ret)))
+
+        prom_counter.labels({'step': 'raw'}).inc(tasks_queued)
         logging.info('got %d tasks to queue', tasks_queued)
 
         tasks = []
         for f in asyncio.as_completed(futures):
-            task = await f
-            # add default resource requirements
-            task.requirements = self._get_resources(task)
+            try:
+                task = await f
+            except Exception:  # already logged in function
+                continue
+            try:
+                # add default resource requirements
+                task.requirements = self._get_resources(task)
+            except Exception:
+                logger.warning('cannot get task resources for %s.%s', task.dataset.dataset_id, task.task_id, exc_info=True)
+                continue
             tasks.append(task)
 
+        tasks_queued2 = len(tasks)
+        prom_counter.labels({'step': 'processed'}).inc(tasks_queued2)
+        logging.info('got %d Task objects to queue', tasks_queued2)
+
         return tasks
 
     async def _convert_to_task(self, task):
         """Convert from basic task dict to a Task object"""
-        d = deepcopy(await self.dataset_lookup(task['dataset_id']))
-        # don't bother looking up the job status - trust that if we got a task, we're in processing
-        j = Job(dataset=d, job_id=task['job_id'], job_index=task['job_index'], status=JOB_STATUS_START)
-        t = Task(
-            dataset=d,
-            job=j,
-            task_id=task['task_id'],
-            task_index=task['task_index'],
-            instance_id=task['instance_id'],
-            name=task['name'],
-            depends=task['depends'],
-            requirements=task['requirements'],
-            status=task['status'],
-            site=self.site,
-            stats={},
-            task_files=[],
-        )
-        await t.load_task_files_from_api(self.rest_client)
+        try:
+            d = deepcopy(await self.dataset_lookup(task['dataset_id']))
+            # don't bother looking up the job status - trust that if we got a task, we're in processing
+            j = Job(dataset=d, job_id=task['job_id'], job_index=task['job_index'], status=JOB_STATUS_START)
+            t = Task(
+                dataset=d,
+                job=j,
+                task_id=task['task_id'],
+                task_index=task['task_index'],
+                instance_id=task['instance_id'],
+                name=task['name'],
+                depends=task['depends'],
+                requirements=task['requirements'],
+                status=task['status'],
+                site=self.site,
+                stats={},
+                task_files=[],
+            )
+            await t.load_task_files_from_api(self.rest_client)
 
-        # load some config defaults
-        config = t.dataset.config
-        if (not config['options'].get('site_temp','')) and self.cfg['queue'].get('site_temp', ''):
-            config['options']['site_temp'] = self.cfg['queue']['site_temp']
-        add_default_options(config['options'])
+            # load some config defaults
+            config = t.dataset.config
+            if (not config['options'].get('site_temp','')) and self.cfg['queue'].get('site_temp', ''):
+                config['options']['site_temp'] = self.cfg['queue']['site_temp']
+            add_default_options(config['options'])
+
+        except Exception:
+            logger.warning('Error converting task dict to task: %s.%s', task['dataset_id'], task['task_id'], exc_info=True)
+            raise
 
         return t
 
@@ -188,8 +228,8 @@ class BaseGrid:
         """
         resource = deepcopy(Resources.defaults)
         for k in resource:
-            if isinstance(resource[k],list):
-                resource[k] = len(resource[k])
+            if isinstance(resource[k], list):
+                resource[k] = len(resource[k])  # type: ignore
         values = {}
         try:
             for k in task.requirements:
@@ -212,6 +252,28 @@ class BaseGrid:
         resource.update(values)
         return resource
 
+    @AsyncPromWrapper(lambda self: self.prometheus.gauge('iceprod_grid_tasks', 'IceProd grid tasks IceProd thinks are on the queue'))
+    async def get_tasks_on_queue(self, prom_gauge) -> list:
+        """
+        Get all tasks that are "assigned" to this queue.
+
+        Returns:
+            list of tasks
+        """
+        args = {
+            'status': 'queued|processing',
+            'site': self.site,
+            'keys': 'dataset_id|task_id|instance_id|status|status_changed',
+        }
+        try:
+            tasks = await self.rest_client.request('GET', '/tasks', args)
+            ret = tasks['tasks']
+            prom_gauge.set(len(ret))
+            return ret
+        except requests.exceptions.HTTPError:
+            logger.warning('cannot get tasks on queue', exc_info=True)
+            return []
+
     # Task Actions #
 
     async def _upload_log(self, task: GridTask, name: str, data: str):
@@ -263,6 +325,7 @@ class BaseGrid:
         except requests.exceptions.HTTPError:
             logger.warning('cannot upload stats', exc_info=True)
 
+    @AsyncPromTimer(lambda self: self.prometheus.histogram('iceprod_grid_task_idle', 'IceProd grid.task_idle calls', buckets=HistogramBuckets.MINUTE))
     async def task_idle(self, task: GridTask):
         """
         Tell IceProd API a task is now idle on the queue (put back to "queue" status).
@@ -283,12 +346,14 @@ class BaseGrid:
             if e.response.status_code != 404:
                 raise
 
-    async def task_processing(self, task: GridTask):
+    @AsyncPromTimer(lambda self: self.prometheus.histogram('iceprod_grid_task_processing', 'IceProd grid.task_processing calls', buckets=HistogramBuckets.MINUTE))
+    async def task_processing(self, task: GridTask, site: str | None = None):
         """
         Tell IceProd API a task is now processing (put in the "processing" status).
 
         Args:
             task: IceProd task info
+            site: computing site the task is running at
         """
         if not task.task_id or not task.instance_id:
             raise RuntimeError("Either task_id or instance_id is empty")
@@ -296,18 +361,22 @@ class BaseGrid:
         args = {
             'instance_id': task.instance_id,
         }
+        if site:
+            args['site'] = site
         try:
             await self.rest_client.request('POST', f'/tasks/{task.task_id}/task_actions/processing', args)
         except requests.exceptions.HTTPError as e:
             if e.response.status_code != 404:
                 raise
 
-    async def task_reset(self, task: GridTask, reason: str | None = None):
+    @AsyncPromTimer(lambda self: self.prometheus.histogram('iceprod_grid_task_reset', 'IceProd grid.task_reset calls', buckets=HistogramBuckets.MINUTE))
+    async def task_reset(self, task: GridTask, reason: str | None = None, stats: dict | None = None):
         """
         Tell IceProd API a task should be reset back to the "waiting" status.
 
         Args:
             task: IceProd task info
+            stats: task resource statistics
             reason: A reason for failure
         """
         if not task.task_id or not task.instance_id:
@@ -316,6 +385,10 @@ class BaseGrid:
         args = {
             'instance_id': task.instance_id,
         }
+        if stats:
+            args['resources'] = stats.get('resources', {})
+            if site := stats.get('site'):
+                args['site'] = site
         if reason:
             args['reason'] = reason
         try:
@@ -327,6 +400,7 @@ class BaseGrid:
             if reason:
                 await self._upload_log(task, 'stdlog', reason)
 
+    @AsyncPromTimer(lambda self: self.prometheus.histogram('iceprod_grid_task_failure', 'IceProd grid.task_failure calls', buckets=HistogramBuckets.MINUTE))
     async def task_failure(self, task: GridTask, reason: str | None = None, stats: dict | None = None, stdout: Path | None = None, stderr: Path | None = None):
         """
         Tell IceProd API a task should be put in the "failed" status.
@@ -354,6 +428,8 @@ class BaseGrid:
             args['reason'] = reason
         if stats:
             args['resources'] = stats.get('resources', {})
+            if site := stats.get('site'):
+                args['site'] = site
         try:
             await self.rest_client.request('POST', f'/tasks/{task.task_id}/task_actions/failed', args)
         except requests.exceptions.HTTPError as e:
@@ -369,6 +445,7 @@ class BaseGrid:
             if reason:
                 await self._upload_log(task, 'stdlog', reason)
 
+    @AsyncPromTimer(lambda self: self.prometheus.histogram('iceprod_grid_task_success', 'IceProd grid.task_success calls', buckets=HistogramBuckets.MINUTE))
     async def task_success(self, task: GridTask, stats: dict | None = None, stdout: Path | None = None, stderr: Path | None = None):
         """
         Tell IceProd API a task was successfully completed.
@@ -391,6 +468,8 @@ class BaseGrid:
         }
         if stats:
             resources = stats.get('resources', {})
+            if site := stats.get('site'):
+                args['site'] = site
             if 'time' in resources:
                 args['time_used'] = resources['time']*3600.
 
