@@ -7,37 +7,40 @@ It has been broken down into several sub-handlers for easier maintenance.
 """
 
 from collections import defaultdict
-from datetime import datetime, timedelta
+import dataclasses as dc
+from datetime import datetime, timedelta, UTC
 import functools
 import importlib.resources
 import logging
 import os
 import random
 import re
+from typing import Any
 from urllib.parse import urlencode
 
 from iceprod.core.jsonUtil import json_encode
 
 from cachetools.func import ttl_cache
+from prometheus_client import Info, start_http_server
 import tornado.web
 import tornado.httpserver
 import tornado.gen
 import jwt
 import tornado.concurrent
 from rest_tools.client import RestClient, ClientCredentialsAuth
-from rest_tools.server import catch_error, RestServer, RestHandlerSetup, RestHandler, OpenIDLoginHandler
-from rest_tools import telemetry as wtt
-from wipac_dev_tools import from_environment
+from rest_tools.server import catch_error, RestServer, RestHandlerSetup, RestHandler, OpenIDCookieHandlerMixin, OpenIDLoginHandler
+from rest_tools.server.session import SessionMixin, Session
+from wipac_dev_tools import from_environment_as_dataclass
 
-import iceprod
+from iceprod.util import VERSION_STRING
+from iceprod.prom_utils import AsyncMonitor, PromRequestMixin
 from iceprod.roles_groups import GROUPS
 from iceprod.core.config import CONFIG_SCHEMA as DATASET_SCHEMA
 from iceprod.server.config import CONFIG_SCHEMA as SERVER_SCHEMA
 import iceprod.core.functions
 from iceprod.server import documentation
-from iceprod.server.module import FakeStatsClient, StatsClientIgnoreErrors
 import iceprod.server.states
-from iceprod.server.util import nowstr
+from iceprod.server.util import datetime2str, nowstr
 
 logger = logging.getLogger('website')
 
@@ -98,16 +101,140 @@ def eval_expression(token, e):
     return [r for r in ret if r]
 
 
-class TokenStorageMixin:
+class LoginMixin(SessionMixin, OpenIDCookieHandlerMixin, RestHandler):  # type: ignore[misc]
+    """
+    Store/load current user's `OpenIDLoginHandler` tokens in Redis.
+    """
+    def get_current_user(self) -> str | None:
+        """Get the current username from the cookie"""
+        try:
+            username = self.get_secure_cookie('iceprod_username')
+            if not username:
+                raise RuntimeError('missing iceprod_username cookie')
+            return username.decode('utf-8')
+        except Exception:
+            logger.info('failed to get username')
+        return None
+
+    @property
+    def auth_access_token(self) -> bytes | None:
+        if self.session:
+            ret = self.session.get('access_token', None)
+            if not isinstance(ret, str):
+                logger.warning('bad access token type: not str')
+                return None
+            return ret.encode('utf-8')
+        return None
+
+    @auth_access_token.setter
+    def auth_access_token(self, val: bytes):
+        if self.session:
+            self.session['access_token'] = val.decode('utf-8')
+        else:
+            raise RuntimeError('no valid session')
+
+    @property
+    def auth_refresh_token(self) -> bytes | None:
+        if self.session:
+            ret = self.session.get('refresh_token', None)
+            if not isinstance(ret, str):
+                logger.warning('bad access token type: not str')
+                return None
+            return ret.encode('utf-8')
+        return None
+
+    @auth_refresh_token.setter  # type: ignore[override]
+    def auth_refresh_token(self, val: bytes):
+        if self.session:
+            self.session['refresh_token'] = val.decode('utf-8')
+        else:
+            raise RuntimeError('no valid session')
+
+    @property
+    def auth_groups(self) -> list[str]:
+        assert self.auth
+        if not self.auth_access_token:
+            return []
+        try:
+            data = self.auth.validate(self.auth_access_token)
+        except jwt.ExpiredSignatureError:
+            logger.debug('user access_token expired')
+            return []
+
+        # lookup groups
+        groups: set[str] = set()
+        try:
+            for name in GROUPS:
+                for expression in GROUPS[name]:
+                    ret = eval_expression(data, expression)
+                    groups.update(match.expand(name) for match in ret)
+        except Exception:
+            logger.info('cannot determine groups', exc_info=True)
+        return sorted(groups)
+
+    def store_tokens(
+        self,
+        access_token,
+        access_token_exp,
+        refresh_token=None,
+        refresh_token_exp=None,
+        user_info=None,
+        user_info_exp=None,
+    ):
+        """
+        Store jwt tokens and user info from OpenID-compliant auth source.
+
+        Args:
+            access_token (str): jwt access token
+            access_token_exp (int): access token expiration in seconds
+            refresh_token (str): jwt refresh token
+            refresh_token_exp (int): refresh token expiration in seconds
+            user_info (dict): user info (from id token or user info lookup)
+            user_info_exp (int): user info expiration in seconds
+        """
+        assert self.auth
+        if not user_info:
+            user_info = self.auth.validate(access_token)
+        username = user_info.get('preferred_username')
+        if not username:
+            username = user_info.get('upn')
+        if not username:
+            raise tornado.web.HTTPError(400, reason='no username in token')
+
+        data = {
+            'access_token': access_token,
+            'refresh_token': refresh_token,
+        }
+        self._session_mgr.set(username, data)
+
+        self.set_secure_cookie('iceprod_username', username, expires_days=30)
+
+    def clear_tokens(self):
+        """
+        Clear token data, usually on logout.
+        """
+        username = self.current_user
+        self.clear_cookie('iceprod_username')
+        if username:
+            self._session_mgr.delete_session(username)
+
+
+class TokenStorageMixin(OpenIDCookieHandlerMixin, RestHandler):
     """
     Store/load current user's `OpenIDLoginHandler` tokens in iceprod credentials API.
     """
+    def initialize(self, cred_rest_client: RestClient, full_url: str, **kwargs):  # type: ignore
+        super().initialize(**kwargs)
+        self.cred_rest_client = cred_rest_client
+        self.full_url = full_url
+
     def get_current_user(self):
         return None
 
     async def get_current_user_async(self):
         """Get the current user, and set auth-related attributes."""
         try:
+            assert self.auth
             username = self.get_secure_cookie('iceprod_username')
             if not username:
                 return None
@@ -162,6 +289,7 @@ class TokenStorageMixin:
             user_info (dict): user info (from id token or user info lookup)
             user_info_exp (int): user info expiration in seconds
         """
+        assert self.auth
         if not user_info:
             user_info = self.auth.validate(access_token)
         username = user_info.get('preferred_username')
@@ -188,55 +316,49 @@ class TokenStorageMixin:
         self.clear_cookie('iceprod_username')
 
 
-class Login(TokenStorageMixin, OpenIDLoginHandler):
-    def initialize(self, cred_rest_client=None, full_url=None, **kwargs):
-        super().initialize(**kwargs)
-        self.cred_rest_client = cred_rest_client
-        self.full_url = full_url
+class Login(LoginMixin, PromRequestMixin, OpenIDLoginHandler):  # type: ignore
+    pass
 
 
-class PublicHandler(TokenStorageMixin, RestHandler):
+class PublicHandler(LoginMixin, PromRequestMixin, RestHandler):
     """Default Handler"""
-    def initialize(self, statsd=None, rest_api=None, cred_rest_client=None, system_rest_client=None, full_url=None, **kwargs):
+    def initialize(self, rest_api, cred_rest_client, system_rest_client, **kwargs):  # type: ignore
         """
         Get some params from the website module
 
-        :param statsd: statsd client
         :param rest_api: the rest api url
         :param cred_rest_client: the rest api url for the cred service
         :param system_rest_client: the rest client for the system role
-        :param full_url: the full base url for the website
         """
         super().initialize(**kwargs)
-        self.statsd = statsd
         self.rest_api = rest_api
         self.cred_rest_client = cred_rest_client
         self.system_rest_client = system_rest_client
-        self.rest_client = None
-        self.full_url = full_url
+        self.rest_client: RestClient | None = None
 
-    def get_template_namespace(self):
+    def get_template_namespace(self) -> dict[str, Any]:
         namespace = super().get_template_namespace()
-        namespace['version'] = iceprod.__version__
-        namespace['section'] = self.request.uri.lstrip('/').split('?')[0].split('/')[0]
+        namespace['version'] = VERSION_STRING
+        if self.request.uri:
+            namespace['section'] = self.request.uri.lstrip('/').split('?')[0].split('/')[0]
         namespace['json_encode'] = json_encode
         namespace['states'] = iceprod.server.states
         namespace['rest_api'] = self.rest_api
         return namespace
 
-    async def get_current_user_async(self):
-        ret = await super().get_current_user_async()
+    async def get_current_user_async(self) -> str | None:
         try:
-            if ret:
+            if self.current_user and self.auth_access_token:
                 self.rest_client = RestClient(self.rest_api, self.auth_access_token, timeout=50, retries=1)
+            elif not self.current_user:
+                logger.info('no current user')
+            else:
+                logger.info('no access token')
         except Exception:
-            pass
+            logger.info('failed to create user rest client', exc_info=True)
+        return self.current_user
 
-        self.current_user = ret
-        return ret
-
-    @wtt.evented(all_args=True)
-    def write_error(self, status_code=500, **kwargs):
+    def write_error(self, status_code: int = 500, **kwargs) -> None:
         """Write out custom error page."""
         self.set_status(status_code)
         if status_code >= 500:
@@ -258,7 +380,6 @@ class Default(PublicHandler):
     async def get(self):
         # try to get the user, if available
         await self.get_current_user_async()
-        self.statsd.incr('default')
         self.render('main.html')
 
 
@@ -279,7 +400,6 @@ class Submit(PublicHandler):
     @authenticated
     async def get(self):
         logger.info('here')
-        self.statsd.incr('submit')
         token = self.auth_access_token
         groups = self.auth_groups
         default_config = {
@@ -296,6 +416,7 @@ class Submit(PublicHandler):
         render_args = {
             'passkey':token,
             'edit':False,
+            'dataset': '',
             'dataset_id':'',
             'config':default_config,
             'groups':groups,
@@ -308,7 +429,7 @@ class Config(PublicHandler):
     """Handle /config urls"""
     @authenticated
     async def get(self):
-        self.statsd.incr('config')
+        assert self.rest_client
         dataset_id = self.get_argument('dataset_id',default=None)
         if not dataset_id:
             self.write_error(400,message='must provide dataset_id')
@@ -323,6 +444,7 @@ class Config(PublicHandler):
         render_args = {
             'edit':edit,
             'passkey':passkey,
+            'dataset': dataset.get('dataset',''),
             'dataset_id':dataset_id,
             'config':config,
             'description':dataset.get('description',''),
@@ -339,7 +461,7 @@ class DatasetBrowse(PublicHandler):
 
     @authenticated
     async def get(self):
-        self.statsd.incr('dataset_browse')
+        assert self.rest_client
         usernames = await self.get_usernames()
         filter_options = {
             'status': ['processing', 'suspended', 'errors', 'complete', 'truncated'],
@@ -375,8 +497,7 @@ class Dataset(PublicHandler):
     """Handle /dataset urls"""
     @authenticated
     async def get(self, dataset_id):
-        self.statsd.incr('dataset')
-
+        assert self.rest_client
         if dataset_id.isdigit():
             try:
                 d_num = int(dataset_id)
@@ -430,9 +551,8 @@ class TaskBrowse(PublicHandler):
     """Handle /task urls"""
     @authenticated
     async def get(self, dataset_id):
-        self.statsd.incr('task_browse')
+        assert self.rest_client
         status = self.get_argument('status',default=None)
-
         if status:
             tasks = await self.rest_client.request('GET','/datasets/{}/tasks?status={}'.format(dataset_id,status))
             for t in tasks:
@@ -450,9 +570,8 @@ class Task(PublicHandler):
     """Handle /task urls"""
     @authenticated
     async def get(self, dataset_id, task_id):
-        self.statsd.incr('task')
+        assert self.rest_client
         status = self.get_argument('status', default=None)
-
         passkey = self.auth_access_token
 
         dataset = await self.rest_client.request('GET', '/datasets/{}'.format(dataset_id))
@@ -483,9 +602,8 @@ class JobBrowse(PublicHandler):
     """Handle /job urls"""
     @authenticated
     async def get(self, dataset_id):
-        self.statsd.incr('job')
+        assert self.rest_client
         status = self.get_argument('status',default=None)
-
         passkey = self.auth_access_token
 
         jobs = await self.rest_client.request('GET', '/datasets/{}/jobs'.format(dataset_id))
@@ -501,9 +619,8 @@ class Job(PublicHandler):
     """Handle /job urls"""
     @authenticated
     async def get(self, dataset_id, job_id):
-        self.statsd.incr('job')
+        assert self.rest_client
         status = self.get_argument('status',default=None)
-
         passkey = self.auth_access_token
 
         dataset = await self.rest_client.request('GET', '/datasets/{}'.format(dataset_id))
@@ -522,7 +639,6 @@ class Documentation(PublicHandler):
     async def get(self, url):
         # try to get the user, if available
         await self.get_current_user_async()
-        self.statsd.incr('documentation')
         doc_path = str(importlib.resources.files('iceprod.server')/'data'/'docs')
         full_path = os.path.join(doc_path, url)
         if not full_path.startswith(doc_path):
@@ -536,7 +652,7 @@ class Documentation(PublicHandler):
 class Log(PublicHandler):
     @authenticated
     async def get(self, dataset_id, log_id):
-        self.statsd.incr('log')
+        assert self.rest_client
         ret = await self.rest_client.request('GET','/datasets/{}/logs/{}'.format(dataset_id, log_id))
         log_text = ret['data']
         html = '<html><head><title>' + ret['name'] + '</title></head><body>'
@@ -552,7 +668,6 @@ class Help(PublicHandler):
     async def get(self):
         # try to get the user, if available
         await self.get_current_user_async()
-        self.statsd.incr('help')
         self.render('help.html')
 
 
@@ -562,7 +677,6 @@ class Other(PublicHandler):
     async def get(self):
         # try to get the user, if available
         await self.get_current_user_async()
-        self.statsd.incr('other')
         path = self.request.path
         self.set_status(404)
         self.render('404.html', path=path)
@@ -572,7 +686,6 @@ class Profile(PublicHandler):
     """Handle user profile page"""
     @authenticated
     async def get(self):
-        self.statsd.incr('profile')
         username = self.current_user
         groups = self.auth_groups
         group_creds = {}
@@ -596,7 +709,7 @@ class Profile(PublicHandler):
             }
             if self.auth_refresh_token:
                 args['refresh_token'] = self.auth_refresh_token
-                args['expiration'] = (datetime.utcnow() + timedelta(days=30)).isoformat()
+                args['expiration'] = datetime2str(datetime.now(UTC) + timedelta(days=30))
             await self.cred_rest_client.request('POST', f'/users/{username}/credentials', args)
 
         else:
@@ -634,7 +747,6 @@ class Profile(PublicHandler):
 class Logout(PublicHandler):
     @catch_error
     async def get(self):
-        self.statsd.incr('logout')
         self.clear_tokens()
         self.current_user = None
         self.request.uri = '/'  # for login redirect, fake the main page
@@ -674,27 +786,32 @@ class HealthHandler(PublicHandler):
         self.write(status)
 
 
+@dc.dataclass(frozen=True)
+class DefaultConfig:
+    HOST : str = 'localhost'
+    PORT : int = 8080
+    DEBUG : bool = False
+    OPENID_URL : str = ''
+    OPENID_AUDIENCE : str = ''
+    ICEPROD_WEB_URL : str = 'https://iceprod2.icecube.wisc.edu'
+    ICEPROD_API_ADDRESS : str = 'https://iceprod2-api.icecube.wisc.edu'
+    ICEPROD_API_CLIENT_ID : str = ''
+    ICEPROD_API_CLIENT_SECRET : str = ''
+    ICEPROD_CRED_ADDRESS : str = 'https://credentials.iceprod.icecube.aq'
+    ICEPROD_CRED_CLIENT_ID : str = ''
+    ICEPROD_CRED_CLIENT_SECRET : str = ''
+    REDIS_HOST : str = 'localhost'
+    REDIS_USER : str = ''
+    REDIS_PASSWORD : str = ''
+    REDIS_TLS : bool = False
+    COOKIE_SECRET : str = ''
+    PROMETHEUS_PORT : int = 0
+    CI_TESTING : str = ''
+
+
 class Server:
     def __init__(self):
-        default_config = {
-            'HOST': 'localhost',
-            'PORT': 8080,
-            'DEBUG': False,
-            'OPENID_URL': '',
-            'OPENID_AUDIENCE': '',
-            'ICEPROD_WEB_URL': 'https://iceprod2.icecube.wisc.edu',
-            'ICEPROD_API_ADDRESS': 'https://iceprod2-api.icecube.wisc.edu',
-            'ICEPROD_API_CLIENT_ID': '',
-            'ICEPROD_API_CLIENT_SECRET': '',
-            'ICEPROD_CRED_ADDRESS': 'https://credentials.iceprod.icecube.aq',
-            'ICEPROD_CRED_CLIENT_ID': '',
-            'ICEPROD_CRED_CLIENT_SECRET': '',
-            'COOKIE_SECRET': '',
-            'STATSD_ADDRESS': '',
-            'STATSD_PREFIX': 'rest_api',
-            'CI_TESTING': '',
-        }
-        config = from_environment(default_config)
+        config = from_environment_as_dataclass(DefaultConfig)
 
         # get package data
         static_path = str(importlib.resources.files('iceprod.website')/'data'/'www')
@@ -707,25 +824,25 @@ class Server:
             raise Exception('bad template path')
 
         # set IceProd REST API
-        if config['ICEPROD_API_ADDRESS']:
-            rest_address = config['ICEPROD_API_ADDRESS']
+        if config.ICEPROD_API_ADDRESS:
+            rest_address = config.ICEPROD_API_ADDRESS
         else:
             raise RuntimeError('ICEPROD_API_ADDRESS not specified')
 
         rest_config = {
-            'debug': config['DEBUG'],
-            'server_header': 'IceProd/' + iceprod.__version__,
+            'debug': config.DEBUG,
+            'server_header': 'IceProd/' + VERSION_STRING,
         }
 
-        if config['OPENID_URL']:
-            logging.info(f'enabling auth via {config["OPENID_URL"]} for aud "{config["OPENID_AUDIENCE"]}"')
+        if config.OPENID_URL:
+            logging.info(f'enabling auth via {config.OPENID_URL} for aud "{config.OPENID_AUDIENCE}"')
             rest_config.update({
                 'auth': {
-                    'openid_url': config['OPENID_URL'],
-                    'audience': config['OPENID_AUDIENCE'],
+                    'openid_url': config.OPENID_URL,
+                    'audience': config.OPENID_AUDIENCE,
                 }
             })
-        elif config['CI_TESTING']:
+        elif config.CI_TESTING:
             rest_config.update({
                 'auth': {
                     'secret': 'secret',
@@ -734,70 +851,72 @@ class Server:
         else:
             raise RuntimeError('OPENID_URL not specified, and CI_TESTING not enabled!')
 
-        statsd = FakeStatsClient()
-        if config['STATSD_ADDRESS']:
-            try:
-                addr = config['STATSD_ADDRESS']
-                port = 8125
-                if ':' in addr:
-                    addr,port = addr.split(':')
-                    port = int(port)
-                statsd = StatsClientIgnoreErrors(addr, port=port, prefix=config['STATSD_PREFIX'])
-            except Exception:
-                logger.warning('failed to connect to statsd: %r', config['STATSD_ADDRESS'], exc_info=True)
+        # enable monitoring
+        self.prometheus_port = config.PROMETHEUS_PORT if config.PROMETHEUS_PORT > 0 else None
+        self.async_monitor = None
 
-        if config['ICEPROD_CRED_CLIENT_ID'] and config['ICEPROD_CRED_CLIENT_SECRET']:
-            logging.info(f'enabling auth via {config["OPENID_URL"]} for aud "{config["OPENID_AUDIENCE"]}"')
+        if config.ICEPROD_CRED_CLIENT_ID and config.ICEPROD_CRED_CLIENT_SECRET:
+            logging.info(f'enabling auth via {config.OPENID_URL} for aud "{config.OPENID_AUDIENCE}"')
             cred_client = ClientCredentialsAuth(
-                address=config['ICEPROD_CRED_ADDRESS'],
-                token_url=config['OPENID_URL'],
-                client_id=config['ICEPROD_CRED_CLIENT_ID'],
-                client_secret=config['ICEPROD_CRED_CLIENT_SECRET'],
+                address=config.ICEPROD_CRED_ADDRESS,
+                token_url=config.OPENID_URL,
+                client_id=config.ICEPROD_CRED_CLIENT_ID,
+                client_secret=config.ICEPROD_CRED_CLIENT_SECRET,
             )
-        elif config['CI_TESTING']:
-            cred_client = RestClient(config['ICEPROD_CRED_ADDRESS'], timeout=1, retries=0)
+        elif config.CI_TESTING:
+            cred_client = RestClient(config.ICEPROD_CRED_ADDRESS, timeout=1, retries=0)
         else:
             raise RuntimeError('ICEPROD_CRED_CLIENT_ID or ICEPROD_CRED_CLIENT_SECRET not specified, and CI_TESTING not enabled!')
 
         handler_args = RestHandlerSetup(rest_config)
         handler_args['cred_rest_client'] = cred_client
+        if config.CI_TESTING:
+            self.session = Session()
+        else:
+            kwargs = {}
+            if config.REDIS_USER:
+                kwargs['username'] = config.REDIS_USER
+            if config.REDIS_PASSWORD:
+                kwargs['password'] = config.REDIS_PASSWORD
+            if config.REDIS_TLS:
+                kwargs['tls'] = True
+            self.session = Session(storage_type='redis', host=config.REDIS_HOST, **kwargs)  # type: ignore
+        handler_args['session'] = self.session
 
-        full_url = config['ICEPROD_WEB_URL']
-        handler_args['full_url'] = full_url
+        full_url = config.ICEPROD_WEB_URL
         login_url = full_url+'/login'
 
         login_handler_args = handler_args.copy()
-        if config['ICEPROD_API_CLIENT_ID'] and config['ICEPROD_API_CLIENT_SECRET']:
+        if config.ICEPROD_API_CLIENT_ID and config.ICEPROD_API_CLIENT_SECRET:
             logging.info('enabling system rest client and website logins"')
             rest_client = ClientCredentialsAuth(
-                address=config['ICEPROD_API_ADDRESS'],
-                token_url=config['OPENID_URL'],
-                client_id=config['ICEPROD_API_CLIENT_ID'],
-                client_secret=config['ICEPROD_API_CLIENT_SECRET'],
+                address=config.ICEPROD_API_ADDRESS,
+                token_url=config.OPENID_URL,
+                client_id=config.ICEPROD_API_CLIENT_ID,
+                client_secret=config.ICEPROD_API_CLIENT_SECRET,
             )
-            login_handler_args['oauth_client_id'] = config['ICEPROD_API_CLIENT_ID']
-            login_handler_args['oauth_client_secret'] = config['ICEPROD_API_CLIENT_SECRET']
+            login_handler_args['oauth_client_id'] = config.ICEPROD_API_CLIENT_ID
+            login_handler_args['oauth_client_secret'] = config.ICEPROD_API_CLIENT_SECRET
             login_handler_args['oauth_client_scope'] = 'offline_access posix profile'
-        elif config['CI_TESTING']:
+        elif config.CI_TESTING:
             logger.info('CI_TESTING: no login for testing')
-            rest_client = RestClient(config['ICEPROD_API_ADDRESS'], timeout=1, retries=0)
+            rest_client = RestClient(config.ICEPROD_API_ADDRESS, timeout=1, retries=0)
         else:
             raise RuntimeError('ICEPROD_API_CLIENT_ID or ICEPROD_API_CLIENT_SECRET not specified, and CI_TESTING not enabled!')
 
         handler_args.update({
-            'statsd': statsd,
             'rest_api': rest_address,
             'system_rest_client': rest_client,
         })
-        if config['COOKIE_SECRET']:
-            cookie_secret = config['COOKIE_SECRET']
+        if config.COOKIE_SECRET:
+            cookie_secret = config.COOKIE_SECRET
             log_cookie_secret = cookie_secret[:4] + 'X'*(len(cookie_secret)-8) + cookie_secret[-4:]
             logger.info('using supplied cookie secret %r', log_cookie_secret)
         else:
             cookie_secret = ''.join(hex(random.randint(0,15))[-1] for _ in range(64))
 
         server = RestServer(
-            debug=config['DEBUG'],
+            debug=config.DEBUG,
             cookie_secret=cookie_secret,
             login_url=login_url,
             template_path=template_path,
@@ -823,12 +942,24 @@ class Server:
         server.add_route('/healthz', HealthHandler, handler_args)
         server.add_route(r"/.*", Other, handler_args)
 
-        server.startup(address=config['HOST'], port=config['PORT'])
+        server.startup(address=config.HOST, port=config.PORT)
 
         self.server = server
 
     async def start(self):
-        pass
+        if self.prometheus_port:
+            logging.info("starting prometheus on %r", self.prometheus_port)
+            start_http_server(self.prometheus_port)
+            i = Info('iceprod', 'IceProd information')
+            i.info({
+                'version': VERSION_STRING,
+                'type': 'website',
+            })
+            self.async_monitor = AsyncMonitor(labels={'type': 'website'})
+            await self.async_monitor.start()
 
     async def stop(self):
         await self.server.stop()
+        if self.async_monitor:
+            await self.async_monitor.stop()
+        self.session.close()
